

/*
Horloge mere LAMBERT TELESECT
Avec ATTiny85 programmé via Arduino.
Base de temps secteur double alternance (100Hz).
http://f5mna.free.fr

Mai 2015       Version 1 F5MNA
Octobre 2016   Version 2 F4GRT
Decembre 2016  Version 2.1 F4GRT -- email:philippe.creis(@)wanadoo(.)fr -- (Enlevez les parenthèses dans l'adresse email)
Ce code est valide pour un ATtiny 45/85 cadencé à 1MHz en interne

                                       +-\_/-+
                              A0 PB5  1|     |8  Vcc
                  SORTIE 1    A3 PB3  2|     |7  PB2 D0 A1    Entrée des signaux carrés 100Hz
                  SORTIE 2    A2 PB4  3|     |6  PB1 D1
                                 GND  4|     |5  PB0 D2       Entrée BP "Avance rapide" 
                                       +-----+        

*/
// -----------------------------------------------------------------------------------------------
const int Top1 = 3 ;              // La sortie 1 est sur la pin PB3
const int Top2 = 4 ;              // La sortie 2 est sur la pin PB4
const long duree = 30000;         // Durée d'activation des sorties lors de l'avance rapide

// Ces variables sont changées en dehors de la boucle principale (vide)
// Elles doivent donc être declarées "volatiles"

volatile int Tps = 60;
volatile int compteur;         // variable compteur des fronts d'horloge
volatile int Temps, i = 0;     // variable compteur des secondes
volatile int Sorties = 0;      // Variable synthese etat des des sorties

// -----------------------------------------------------------------------------------------------
void setup() {
  cli();                      // Interruptions desactivees durant le setup
  pinMode(Top1, OUTPUT);      // La broche PB3 est déclarée comme une  sortie
  pinMode(Top2, OUTPUT);      // La broche PB4 est déclarée comme une  sortie

  GIMSK = 0b00100000;          // On active les interruptions sur changement d'état des Pins
  PCMSK = 0b00000101;          // Les interruptions seront prises en compte sur les entrees PB0 et PB2

  sei();                       // Interruptions activees de nouveau
}

// -----------------------------------------------------------------------------------------------
void loop() {
              // Rien à faire dans la boucle principale
              // Nothing to do in the main loop
}

// -----------------------------------------------------------------------------------------------
ISR (PCINT0_vect) {            // Routine executee sur interruption

  if (bitRead (PINB,0) && !Sorties) {              // Test Bp "Avance Rapide" appuye et sorties inactives
  //if ((PINB & 0b00000001) == 1 && !Sorties) {    // Test Bp "Avance Rapide" appuye et sorties inactives
    AvanceRapide();                  // Alors on execute la boucle "Avance rapide"
  }

  else {			 // Sinon on continue la routine sur interruption
    compteur++;                  // Increment du compteur d'impulsions (En realité, les fronts montants et descendants)
    if (compteur == 200) {       // Si compteur pulses == 200 alors 1 seconde s'est écoulée (200 => 200 fronts soit 100Hz issus d'un
                                 //   redressement double alternance du 50Hz
      ( Temps ++ );              // Increment du compteur des secondes
      compteur = 0;              // On reset le compteur des impulsions
    }

    if (Temps > 0)                  // Si nombre de secondes >0, on désactive les sorties qui seraient à l'état haut
    { PORTB = PORTB & B11100111;    // Mets à 0 les sorties PB3 et PB4
      Sorties = 0;
    }

    if (Temps == Tps) {           // Si Temps == Tps, on va activer une sortie ou l'autre
                                  // en fonction de (i) et remettre à 0 la variable Temps
      Temps = 0;                  // On a changé de minute, on reset la variable temps
                                  // qui contient le nombre de secondes ecoulees                                  
      i ++ ;                      // Si nombre secondes == Tps, on incremente la variable i
                                  // pour l'alternance des sorties
      if (i < 2)                  // Si i = 1, alors on active la sortie 1
      {
        Sorties = 1;                // Une sortie est active
        PORTB = PORTB ^ B00001000;  // On inverse la sortie 2 PB3 (HIGH)
      }
      else {                        // sinon, i=2 et on active la sortie 2
        i = 0;                      // On reset la variable i pour le tour suivant où
                                    // elle repassera à 1 pour la sortie 1
        Sorties = 1;                // Une sortie est active
        PORTB = PORTB ^ B00010000;  // On inverse la sortie 3 PB4 (HIGH)
      }
    }                               
  }
}

// -----------------------------------------------------------------------------------------------
void delai() {                       // routine temporisation courte pour avance rapide
  volatile unsigned int j;
  j = duree;                         // Durée d'activation des sorties lors de l'avance rapide
  do (j--);
  while (j != 0);
}

// -----------------------------------------------------------------------------------------------
void AvanceRapide() {             // Routine appelée sur appui du bouton  "Avance rapide"
                                  // la variable "i" mémorise la dernière sortie activee

  switch (i)	{
    case 0:
      PORTB = PORTB ^ B00001000;  // On inverse la sortie 2 PB3 (LOW --> HIGH)
      delai ();
      i ++ ;
      PORTB = PORTB ^ B00001000;  // On inverse la sortie 2 PB3 (HIGH --> LOW)
      Temps = 0;                  // On a changé de minute en manuel, on reset la variable temps
                                  // qui contient le nombre de secondes ecoulees                                  
      break;

    case 1:
      PORTB = PORTB ^ B00010000;  // On inverse la sortie 3 PB4 (LOW --> HIGH)
      delai ();
      i ++ ;
      PORTB = PORTB ^ B00010000;  // On inverse la sortie 3 PB4 (HIGH --> LOW)
      Temps = 0;                  // On a changé de minute en manuel, on reset la variable temps
                                  // qui contient le nombre de secondes ecoulees                                  
      break;

    case 2:
      PORTB = PORTB ^ B00001000;  // On inverse la sortie 2 PB3 (LOW --> HIGH)
      delai ();
      PORTB = PORTB ^ B00001000;  // On inverse la sortie 2 PB3 (HIGH --> LOW)
      i --;                       // On reset la variable i à 1 pour le tour suivant
      Temps = 0;                  // On a changé de minute en manuel, on reset la variable temps
                                  // qui contient le nombre de secondes ecoulees                                  
      break;
  }
}


